package com.example.nto.controller;

import com.example.nto.entity.Employee;
import com.example.nto.entity.User;
import com.example.nto.service.EmployeeService;
import com.example.nto.service.UserService;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.security.crypto.password.PasswordEncoder; // Импортируйте PasswordEncoder
import org.springframework.web.bind.annotation.*;

import java.util.List;
import java.util.Map;
import java.util.Optional;

@RestController
@RequestMapping("/api")
public class EmployeeController {
    private final EmployeeService employeeService;
    private final UserService userService;
    private final PasswordEncoder passwordEncoder; // Добавленное поле

    public EmployeeController(EmployeeService employeeService, UserService userService, PasswordEncoder passwordEncoder) {
        this.employeeService = employeeService;
        this.userService = userService;
        this.passwordEncoder = passwordEncoder; // Инициализация поля
    }

    @PreAuthorize("hasRole('ADMIN')")
    @GetMapping("/admin/employees")
    public ResponseEntity<List<Employee>> getAllEmployees() {
        List<Employee> employees = employeeService.findAll();
        return ResponseEntity.ok(employees);
    }

    @PostMapping("/register")
    public ResponseEntity<?> registerUser(@RequestBody User user) {
        userService.saveUser(user);
        return ResponseEntity.status(HttpStatus.CREATED).body("User registered successfully");
    }

    @GetMapping("/{login}/auth") // auth
    public ResponseEntity<?> authenticate(@PathVariable String login) {
        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isPresent()) {
            return ResponseEntity.ok("Valid login");
        } else {
            return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login");
        }
    }

    @GetMapping("/{login}/info")
    public ResponseEntity<?> getInfo(@PathVariable String login) {
        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isPresent()) {
            return ResponseEntity.ok(employee.get());
        }
        return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login");
    }

    @PatchMapping("/{login}/open") // open
    public ResponseEntity<?> openDoor(@PathVariable String login, @RequestBody Map<String, Long> payload) {
        Long code = payload.get("value");
        if (code == null) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid payload");
        }

        Optional<Employee> employee = employeeService.findByLogin(login);
        if (employee.isEmpty()) {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid login");
        }

        if (employeeService.validateCode(login, code)) {
            return ResponseEntity.ok("Door opened");
        } else {
            return ResponseEntity.status(HttpStatus.BAD_REQUEST).body("Invalid code");
        }
    }

    @GetMapping("/auth")
    public ResponseEntity<?> authenticate(@RequestParam String username, @RequestParam String password) {
        Optional<User> optionalUser = userService.findByUsername(username); // Исправление
        if (optionalUser.isPresent() && passwordEncoder.matches(password, optionalUser.get().getPassword())) { // Исправление
            return ResponseEntity.ok("Valid login");
        }
        return ResponseEntity.status(HttpStatus.UNAUTHORIZED).body("Invalid login");
    }
}